const createHistoryStateStore = (state = window.location.href) => {
  const setState = (nextState, shouldPushState) => {
    if ("string" != typeof nextState) throw new Error(`non-string href: ${nextState}`);
    if (nextState === state) return state;
    const prevState = state;
    state = nextState;
    "skip" !== shouldPushState && window.history.pushState(null, "", state);
    listenerSet.forEach(listener => listener(state, prevState));
    return state;
  };
  const listenerPopState = () => {
    setState(window.location.href, "skip");
  };
  const listenerHashChange = () => {
    setState(window.location.href);
  };
  const listenerSet = new Set();
  return {
    getState: () => {
      if (0 === listenerSet.size) throw new Error("should subscribe before getState");
      return state;
    },
    setState,
    subscribe: listener => {
      listenerSet.add(listener);
      if (1 === listenerSet.size) {
        window.addEventListener("popstate", listenerPopState);
        window.addEventListener("hashchange", listenerHashChange);
      }
    },
    unsubscribe: listener => {
      listenerSet.delete(listener);
      if (0 === listenerSet.size) {
        window.removeEventListener("popstate", listenerPopState);
        window.removeEventListener("hashchange", listenerHashChange);
      }
    }
  };
};
export { createHistoryStateStore };