import { concatArrayBuffer, deconcatArrayBuffer, fromString, toString } from "./ArrayBuffer";
const MAX_PACKET_HEADER_SIZE = 4294967295;
const HEADER_BYTE_SIZE = 4;
const EMPTY_ARRAY_BUFFER = new ArrayBuffer(0);
const packArrayBufferHeader = headerArrayBuffer => {
  const headerSize = headerArrayBuffer.byteLength;
  if (headerSize > 4294967295) throw new Error(`headerArrayBuffer exceeds max size 4294967295, get: ${headerSize}`);
  const headerSizeDataView = new DataView(new ArrayBuffer(4));
  headerSizeDataView.setUint32(0, headerSize, !1);
  return [ headerSizeDataView.buffer, headerArrayBuffer ];
};
const parseArrayBufferHeader = arrayBufferPair => {
  const headerSize = new DataView(arrayBufferPair.slice(0, 4)).getUint32(0, !1);
  return [ arrayBufferPair.slice(4, 4 + headerSize), 4 + headerSize ];
};
const packArrayBufferPacket = (headerString, payloadArrayBuffer = EMPTY_ARRAY_BUFFER) => concatArrayBuffer([ ...packArrayBufferHeader(fromString(headerString)), payloadArrayBuffer ]);
const parseArrayBufferPacket = arrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(arrayBufferPacket);
  return [ toString(headerArrayBuffer), arrayBufferPacket.slice(payloadOffset) ];
};
const packChainArrayBufferPacket = (arrayBufferList = []) => {
  const headerDataView = new DataView(new ArrayBuffer(4 * arrayBufferList.length));
  arrayBufferList.forEach(({byteLength}, index) => headerDataView.setUint32(4 * index, byteLength, !1));
  return concatArrayBuffer([ ...packArrayBufferHeader(headerDataView.buffer), ...arrayBufferList ]);
};
const parseChainArrayBufferPacket = chainArrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(chainArrayBufferPacket);
  const headerDataView = new DataView(headerArrayBuffer);
  const byteLengthList = [];
  for (let index = 0, indexMax = headerDataView.byteLength / 4; index < indexMax; index++) byteLengthList.push(headerDataView.getUint32(4 * index, !1));
  return deconcatArrayBuffer(chainArrayBufferPacket, byteLengthList, payloadOffset);
};
export { MAX_PACKET_HEADER_SIZE, HEADER_BYTE_SIZE, packArrayBufferHeader, parseArrayBufferHeader, packArrayBufferPacket, parseArrayBufferPacket, packChainArrayBufferPacket, parseChainArrayBufferPacket };