import { catchAsync } from "../error";
const NULL_FUNC = () => {};
const createSaveQueue = ({onError, doSave, beforeSave = NULL_FUNC, afterSave = NULL_FUNC}) => {
  let dataQueue = [];
  let isSaving = !1;
  let savingState = {};
  const setSavingState = nextSavingState => {
    savingState = nextSavingState;
  };
  return {
    clear: () => {
      dataQueue = [];
    },
    add: data => dataQueue.push(data),
    save: () => {
      !isSaving && dataQueue.length && (async () => {
        beforeSave();
        const savingDataQueue = dataQueue;
        dataQueue = [];
        isSaving = !0;
        const {error} = await catchAsync(doSave, savingDataQueue, setSavingState);
        isSaving = !1;
        savingState = {};
        if (error) throw error;
        afterSave();
      })().catch(onError);
    },
    filter: filterFunc => {
      dataQueue = dataQueue.filter(filterFunc);
    },
    getLength: () => dataQueue.length,
    getIsSaving: () => isSaving,
    getSavingState: () => savingState
  };
};
export { createSaveQueue };