const createTreeDepthFirstSearch = getSubNodeListFunc => {
  const unshiftStack = (stack, node) => {
    const nodeList = getSubNodeListFunc(node);
    Array.isArray(nodeList) && stack.unshift(...nodeList);
  };
  return (initialNode, func) => {
    let node;
    const stack = [];
    unshiftStack(stack, initialNode);
    for (;node = stack.shift(); ) {
      if (func(node)) return node;
      unshiftStack(stack, node);
    }
  };
};
const createTreeDepthFirstSearchAsync = getSubNodeListFunc => {
  const unshiftStack = async (stack, node) => {
    const nodeList = await getSubNodeListFunc(node);
    Array.isArray(nodeList) && stack.unshift(...nodeList);
  };
  return async (initialNode, func) => {
    let node;
    const stack = [];
    await unshiftStack(stack, initialNode);
    for (;node = stack.shift(); ) {
      if (await func(node)) return node;
      await unshiftStack(stack, node);
    }
  };
};
const createTreeBreadthFirstSearch = getSubNodeListFunc => {
  const pushStack = (stack, node) => {
    const nodeList = getSubNodeListFunc(node);
    Array.isArray(nodeList) && stack.push(...nodeList);
  };
  return (initialNode, func) => {
    let node;
    const stack = [];
    pushStack(stack, initialNode);
    for (;node = stack.shift(); ) {
      if (func(node)) return node;
      pushStack(stack, node);
    }
  };
};
const createTreeBreadthFirstSearchAsync = getSubNodeListFunc => {
  const pushStack = async (stack, node) => {
    const nodeList = await getSubNodeListFunc(node);
    Array.isArray(nodeList) && stack.push(...nodeList);
  };
  return async (initialNode, func) => {
    let node;
    const stack = [];
    await pushStack(stack, initialNode);
    for (;node = stack.shift(); ) {
      if (await func(node)) return node;
      await pushStack(stack, node);
    }
  };
};
const createTreeBottomUpSearch = getSubNodeListFunc => {
  const pushStack = (stack, node) => {
    const nodeList = getSubNodeListFunc(node);
    return Boolean(Array.isArray(nodeList) && stack.push([ node, nodeList ]));
  };
  return (initialNode, func) => {
    const stack = [];
    pushStack(stack, initialNode);
    for (;0 !== stack.length; ) {
      const [node, subNodeList] = stack[stack.length - 1];
      if (0 === subNodeList.length) {
        stack.pop();
        if (0 !== stack.length && func(node)) return node;
      } else {
        const subNode = subNodeList.shift();
        if (!pushStack(stack, subNode) && func(subNode)) return subNode;
      }
    }
  };
};
const createTreeBottomUpSearchAsync = getSubNodeListFunc => {
  const pushStack = async (stack, node) => {
    const nodeList = await getSubNodeListFunc(node);
    return Boolean(Array.isArray(nodeList) && stack.push([ node, nodeList ]));
  };
  return async (initialNode, func) => {
    const stack = [];
    await pushStack(stack, initialNode);
    for (;0 !== stack.length; ) {
      const [node, subNodeList] = stack[stack.length - 1];
      if (0 === subNodeList.length) {
        stack.pop();
        if (0 !== stack.length && await func(node)) return node;
      } else {
        const subNode = subNodeList.shift();
        if (!await pushStack(stack, subNode) && await func(subNode)) return subNode;
      }
    }
  };
};
const prettyStringifyTree = (initialNode, getSubNodeLevelHasMoreListFunc, func) => {
  const studList = [];
  createTreeDepthFirstSearch(getSubNodeLevelHasMoreListFunc)(initialNode, ([node, level, hasMore]) => {
    studList.length = level;
    0 !== level && (studList[level - 1] = hasMore ? "├─ " : "└─ ");
    func(studList.join(""), node);
    0 !== level && (studList[level - 1] = hasMore ? "│  " : "   ");
  });
};
export { createTreeDepthFirstSearch, createTreeDepthFirstSearchAsync, createTreeBreadthFirstSearch, createTreeBreadthFirstSearchAsync, createTreeBottomUpSearch, createTreeBottomUpSearchAsync, prettyStringifyTree };