const objectSet = (object, key, value) => object[key] !== value ? {
  ...object,
  [key]: value
} : object;
const objectDelete = (object, key) => {
  if (!Object.prototype.hasOwnProperty.call(object, key)) return object;
  const result = {
    ...object
  };
  delete result[key];
  return result;
};
const objectMerge = (object, mergeObject) => {
  for (const [key, value] of Object.entries(mergeObject)) if (object[key] !== value) return {
    ...object,
    ...mergeObject
  };
  return object;
};
const objectMap = (object, mapFunc) => {
  const result = {};
  for (const [key, value] of Object.entries(object)) result[key] = mapFunc(value, key);
  return result;
};
const objectPickKey = (object, keyList) => {
  const result = {};
  for (const key of keyList) Object.prototype.hasOwnProperty.call(object, key) && (result[key] = object[key]);
  return result;
};
const objectFindKey = (object, findEntryFunc) => {
  const entry = Object.entries(object).find(findEntryFunc);
  return entry && entry[0];
};
const objectDeleteUndefined = object => {
  let result;
  for (const [key, value] of Object.entries(object)) if (void 0 === value) {
    void 0 === result && (result = {
      ...object
    });
    delete result[key];
  }
  return result || object;
};
export { objectSet, objectDelete, objectMerge, objectMap, objectPickKey, objectFindKey, objectDeleteUndefined };