import { createAsyncTaskQueue } from "./AsyncTaskQueue";
const createAsyncTaskLane = ({laneSize, selectLane = selectMinLoadLane}) => {
  if (!((laneSize = Number(laneSize)) >= 1)) throw new Error(`invalid laneSize: ${laneSize}`);
  const laneList = [];
  for (let index = 0; index < laneSize; index++) laneList.push({
    index,
    infoQueue: [],
    taskQueue: createAsyncTaskQueue()
  });
  return {
    reset: () => laneList.forEach(({taskQueue, infoQueue}) => {
      taskQueue.reset();
      infoQueue.length = 0;
    }),
    getStatus: isVerbose => laneList.map(({index, infoQueue, taskQueue}) => ({
      index: isVerbose ? index : void 0,
      infoQueue: isVerbose ? infoQueue : infoQueue.length,
      taskQueue: taskQueue.getLength()
    })),
    trimInfoQueue: () => laneList.forEach(({taskQueue, infoQueue}) => {
      infoQueue.length = taskQueue.getLength();
    }),
    pushTask: (asyncTask, info) => {
      const {index, taskQueue, infoQueue} = selectLane(laneList, info);
      const taskPromise = taskQueue.pushTask(asyncTask);
      infoQueue.unshift(info);
      infoQueue.length = taskQueue.getLength();
      return taskPromise;
    }
  };
};
const selectMinLoadLane = laneList => laneList.reduce((o, lane) => o.taskQueue.getLength() > lane.taskQueue.getLength() ? lane : o, laneList[0]);
export { createAsyncTaskLane, selectMinLoadLane };