import { getTimestamp } from "../time";
import { createAsyncTaskQueue } from "./AsyncTaskQueue";
const createAsyncTaskRunner = ({clearRunner = () => {}, resetRunner = (error, taskId) => {}, getTaskId, getTaskInitialState = taskState => taskState, runTask}) => {
  const {reset: resetQueue, getLength: getQueueLength, pushTask} = createAsyncTaskQueue();
  const taskStateMap = new Map();
  const getTaskState = taskId => taskStateMap.get(taskId);
  const updateTaskState = (taskId, nextState) => {
    const state = getTaskState(taskId);
    state && taskStateMap.set(taskId, {
      ...state,
      ...nextState
    });
  };
  const endTask = taskId => taskStateMap.delete(taskId);
  return {
    clear: () => {
      resetQueue();
      taskStateMap.clear();
      return clearRunner();
    },
    getStatus: isVerbose => ({
      taskQueue: getQueueLength(),
      taskStateMap: taskStateMap.size,
      taskStateList: isVerbose ? Array.from(taskStateMap.entries()) : void 0
    }),
    getQueueLength,
    getTaskState,
    startTask: option => {
      const taskId = getTaskId(option);
      return taskStateMap.get(taskId) || ((taskId, option) => {
        const promise = pushTask(() => runTask(taskId, {
          getTaskState,
          updateTaskState
        })).then(result => updateTaskState(taskId, {
          result,
          promise: null,
          endAt: getTimestamp()
        })).catch(error => {
          updateTaskState(taskId, {
            error,
            promise: null,
            endAt: getTimestamp()
          });
          return resetRunner(error, taskId);
        });
        const taskState = getTaskInitialState({
          id: taskId,
          option,
          promise,
          error: null,
          result: null,
          startAt: getTimestamp(),
          endAt: null
        });
        taskStateMap.set(taskId, taskState);
        return taskState;
      })(taskId, option);
    },
    endTask,
    autoEndTask: minEndedTime => {
      const maxEndTime = getTimestamp() - Math.abs(minEndedTime);
      taskStateMap.forEach((task, taskId) => {
        task.endAt && task.endAt <= maxEndTime && endTask(taskId);
      });
    }
  };
};
const createAsyncTaskRunnerCluster = ({runnerList = [], getRunnerByTaskId, selectRunner = selectMinLoadRunner}) => ({
  clear: () => runnerList.map(taskRunner => taskRunner.clear()),
  getStatus: isVerbose => runnerList.map(taskRunner => taskRunner.getStatus(isVerbose)),
  getQueueLength: () => runnerList.map(taskRunner => taskRunner.getQueueLength()),
  getTaskState: taskId => {
    const taskRunner = getRunnerByTaskId(runnerList, taskId);
    return taskRunner && taskRunner.getTaskState(taskId);
  },
  startTask: option => selectRunner(runnerList, option).startTask(option),
  endTask: taskId => {
    const taskRunner = getRunnerByTaskId(runnerList, taskId);
    return taskRunner && taskRunner.endTask(taskId);
  },
  autoEndTask: minEndedTime => runnerList.forEach(taskRunner => taskRunner.autoEndTask(minEndedTime))
});
const selectMinLoadRunner = runnerList => runnerList.reduce((o, taskRunner) => o.getQueueLength() > taskRunner.getQueueLength() ? taskRunner : o, runnerList[0]);
export { createAsyncTaskRunner, createAsyncTaskRunnerCluster, selectMinLoadRunner };