import { getTimestamp } from "../time";
import { getRandomArrayBuffer } from "../math/random";
import { swapObfuscateString } from "../data/function";
import { packArrayBufferPacket, parseArrayBufferPacket } from "../data/ArrayBufferPacket";
const CHAR_CODE_1 = "1".charCodeAt(0);
const calcCode = (size, tokenSize, dataView, seed = 0) => {
  const seedBinaryString = (seed = Math.floor(seed)).toString(2);
  const valueMax = Math.pow(16, tokenSize);
  let index = seed % size;
  let value = dataView.getUint8(index);
  for (let seedIndex = 0, seedIndexMax = seedBinaryString.length; seedIndex < seedIndexMax; seedIndex++) {
    seedBinaryString.charCodeAt(seedIndex) === CHAR_CODE_1 ? index = (index + dataView.getUint8((index + 1) % size)) % size : value *= 16;
    value = (value + dataView.getUint8(index)) % valueMax;
  }
  return (value % Math.pow(2, 4 * tokenSize)).toString(16).padStart(tokenSize, "0");
};
const verifyOption = ({tag = getTimestamp().toString(36), size = 65536, tokenSize = 8, timeGap = 30}) => {
  if (!/^\w*$/.test(tag)) throw new Error(`invalid tag: ${tag}`);
  if (!Number.isInteger(size) || size <= 1024 || size % 32) throw new Error(`invalid size: ${size}`);
  if (!Number.isInteger(tokenSize) || tokenSize > 13 || tokenSize < 2) throw new Error(`invalid tokenSize: ${tokenSize}`);
  if (!Number.isInteger(timeGap) || timeGap < 1) throw new Error(`invalid timeGap: ${timeGap}`);
  return {
    tag,
    size,
    tokenSize,
    timeGap
  };
};
const verifyCheckCode = (timedLookupData, checkCode, timestamp = getTimestamp()) => {
  if ("string" != typeof checkCode || checkCode.length < timedLookupData.tokenSize) throw new Error(`invalid checkCode: ${checkCode}`);
  verifyParsedCheckCode(timedLookupData, parseCheckCode(checkCode), timestamp);
};
const verifyParsedCheckCode = ({tag, size, tokenSize, timeGap, dataView}, [verifyTag, verifyTimestamp, verifyCode], timestamp = getTimestamp()) => {
  if (verifyTag !== tag) throw new Error(`tag mismatch: ${verifyTag}, expect: ${tag}`);
  if (Math.abs(timestamp - verifyTimestamp) > timeGap) throw new Error(`timestamp mismatch: ${verifyTimestamp}, expect: ${timestamp}±${timeGap}`);
  const code = calcCode(size, tokenSize, dataView, verifyTimestamp / timeGap);
  if (code !== verifyCode) throw new Error(`code mismatch: ${verifyCode}, expect: ${code}`);
};
const generateLookupData = option => {
  option = verifyOption(option);
  return {
    ...option,
    dataView: new DataView(getRandomArrayBuffer(option.size))
  };
};
const generateCheckCode = ({tag, size, tokenSize, timeGap, dataView}, timestamp = getTimestamp()) => {
  const code = calcCode(size, tokenSize, dataView, timestamp / timeGap);
  return packCheckCode(tag, timestamp, code);
};
const packCheckCode = (tagString, timestamp, codeString) => swapObfuscateString([ tagString, timestamp.toString(36), codeString ].join("-"));
const parseCheckCode = checkCodeString => {
  const resultList = swapObfuscateString(checkCodeString).split("-");
  resultList[1] = Number.parseInt(resultList[1], 36);
  return resultList;
};
const packDataArrayBuffer = ({tag, size, tokenSize, timeGap, dataView}) => packArrayBufferPacket(JSON.stringify([ tag, size, tokenSize, timeGap ]), dataView.buffer);
const parseDataArrayBuffer = dataArrayBuffer => {
  const [headerString, payloadArrayBuffer] = parseArrayBufferPacket(dataArrayBuffer);
  const [tag, size, tokenSize, timeGap] = JSON.parse(headerString);
  return {
    tag,
    size,
    tokenSize,
    timeGap,
    dataView: new DataView(payloadArrayBuffer)
  };
};
export { verifyOption, verifyCheckCode, verifyParsedCheckCode, generateLookupData, generateCheckCode, packCheckCode, parseCheckCode, packDataArrayBuffer, parseDataArrayBuffer };