import { global } from "../env/global";
const CLOCK_PER_SECOND = 1e3;
const CLOCK_TO_SECOND = .001;
const clock = (() => {
  try {
    const {performance} = global;
    const clock = () => performance.now();
    if (clock() <= clock()) return clock;
  } catch (error) {}
  try {
    const {process} = global;
    const clock = () => {
      const [seconds, nanoseconds] = process.hrtime();
      return 1e3 * seconds + 1e-6 * nanoseconds;
    };
    if (clock() <= clock()) return clock;
  } catch (error) {}
  return Date.now;
})();
const getTimestamp = () => Math.floor(.001 * Date.now());
const setTimeoutAsync = (wait = 0) => new Promise(resolve => setTimeout(resolve, wait));
const [requestFrameUpdate, cancelFrameUpdate] = global.requestAnimationFrame ? [ global.requestAnimationFrame, global.cancelAnimationFrame ] : [ func => setTimeout(func, 1e3 / 60), clearTimeout ];
const createTimer = ({func, delay, queueTask = setTimeout, cancelTask = clearTimeout}) => {
  let token = null;
  const update = () => {
    if (token) {
      token = queueTask(update, delay);
      func();
    }
  };
  const start = () => {
    token || (token = queueTask(update, delay));
  };
  const stop = () => {
    if (token) {
      cancelTask(token);
      token = null;
    }
  };
  return {
    start,
    stop,
    isActive: () => Boolean(token),
    getDelay: () => delay,
    setDelay: nextDelay => {
      if (nextDelay !== delay) {
        delay = nextDelay;
        stop();
        start();
      }
    }
  };
};
const createStepper = (prevTime = clock()) => () => {
  const nextTime = clock();
  const result = nextTime - prevTime;
  prevTime = nextTime;
  return result;
};
export { CLOCK_PER_SECOND, CLOCK_TO_SECOND, clock, getTimestamp, setTimeoutAsync, requestFrameUpdate, cancelFrameUpdate, createTimer, createStepper };