import { Stats } from "fs";
import { dirname } from "path";
import { statAsync, mkdirAsync, rmdirAsync, renameAsync, unlinkAsync, copyFileAsync } from "./function";
const ERROR_STAT = new Stats(-1, -1, 0, -1, -1, -1, 0, 0, 0, 0, 0, 0, 0, 0);
const FILE_TYPE = {
  File: "File",
  Directory: "Directory",
  Other: "Other",
  Error: "Error"
};
const onStatError = error => ERROR_STAT;
const onTrimError = error => {
  if (error && error.code === ERROR_NON_EMPTY) return ERROR_NON_EMPTY;
  throw error;
};
const ERROR_NON_EMPTY = "ENOTEMPTY";
const getPathStat = path => statAsync(path).catch(onStatError);
const getPathTypeFromStat = stat => stat.isDirectory() ? FILE_TYPE.Directory : stat.isFile() ? FILE_TYPE.File : stat !== ERROR_STAT ? FILE_TYPE.Other : FILE_TYPE.Error;
const createDirectory = async (path, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathStat(path));
  if (!pathStat.isDirectory()) {
    if (pathStat !== ERROR_STAT) throw new Error(`path already taken by non-directory: ${path}`);
    await createDirectory(dirname(path));
    await mkdirAsync(path);
  }
};
const trimDirectory = async (path, maxLevel = 1, pathStat) => {
  if (maxLevel) {
    void 0 === pathStat && (pathStat = await getPathStat(path));
    if (pathStat !== ERROR_STAT) {
      if (!pathStat.isDirectory()) throw new Error(`path taken by non-directory: ${path}`);
      if (await rmdirAsync(path).catch(onTrimError) === ERROR_NON_EMPTY) return;
    }
    await trimDirectory(dirname(path), maxLevel - 1);
  }
};
const movePath = async (pathFrom, pathTo, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathStat(pathFrom));
  if (pathStat === ERROR_STAT) throw new Error(`missing path from ${pathFrom}`);
  return renameAsync(pathFrom, pathTo);
};
const copyPath = async (pathFrom, pathTo, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathStat(pathFrom));
  return (!pathStat.isDirectory() || !(await getPathStat(pathTo)).isDirectory()) && (pathStat.isDirectory() ? mkdirAsync(pathTo) : copyFileAsync(pathFrom, pathTo));
};
const deletePath = async (path, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathStat(path));
  return pathStat.isDirectory() ? rmdirAsync(path) : unlinkAsync(path);
};
export { ERROR_STAT, FILE_TYPE, getPathStat, getPathTypeFromStat, createDirectory, trimDirectory, movePath, copyPath, deletePath };