import { dirname } from "path";
import { watch } from "fs";
import { throttle } from "../../common/function";
import { createHub } from "../../common/module/Event";
import { statAsync, nearestExistAsync } from "./function";
const getNull = () => null;
const createFileWatcher = ({wait = 250, persistent = !1}) => {
  const hub = createHub();
  let watcherPath;
  let watcherPathStat;
  let watcher;
  let watcherUpper;
  let targetPath;
  let prevTargetStat;
  const emitThrottled = throttle(async () => {
    const targetStat = await statAsync(targetPath).catch(getNull);
    const isPathChange = Boolean(prevTargetStat) !== Boolean(targetStat);
    targetStat || targetPath !== watcherPath || await setupWatch();
    if (!isPathChange && !targetStat) return;
    const changeState = {
      targetPath,
      isPathChange,
      targetStat
    };
    prevTargetStat = targetStat;
    hub.send(changeState);
  }, wait);
  const onErrorEvent = async (...args) => {
    await setupWatch();
    onChangeEvent("rename");
  };
  const onChangeEvent = (changeType, path) => {
    emitThrottled(changeType, path);
  };
  const clearWatch = () => {
    watcher && watcher.close();
    watcherUpper && watcherUpper.close();
    watcherPath = null;
    watcherPathStat = null;
    watcher = null;
    watcherUpper = null;
  };
  const setupWatch = async () => {
    const targetStat = await statAsync(targetPath).catch(getNull);
    if (targetStat) {
      clearWatch();
      watcherPath = targetPath;
      watcherPathStat = targetStat;
      (watcher = watch(targetPath, {
        persistent,
        recursive: !1
      })).addListener("error", onErrorEvent);
      watcher.addListener("change", onChangeEvent);
      if (targetStat.isDirectory()) {
        (watcherUpper = watch(dirname(targetPath))).addListener("error", getNull);
        watcherUpper.addListener("change", onChangeEvent);
      }
    } else {
      const nearestExistPath = await nearestExistAsync(targetPath);
      if (nearestExistPath === watcherPath) return;
      clearWatch();
      watcherPath = nearestExistPath;
      watcherPathStat = targetStat;
      (watcher = watch(nearestExistPath, {
        persistent,
        recursive: !1
      })).addListener("error", onErrorEvent);
      watcher.addListener("change", onErrorEvent);
    }
  };
  return {
    clear: () => {
      clearWatch();
      hub.clear();
      prevTargetStat = null;
    },
    setup: async path => {
      clearWatch();
      targetPath = path;
      await setupWatch();
      prevTargetStat = watcherPath === targetPath ? watcherPathStat : null;
    },
    subscribe: listener => hub.subscribe(listener),
    unsubscribe: listener => hub.unsubscribe(listener)
  };
};
export { createFileWatcher };