import { resolve as resolvePath, normalize, dirname, sep } from "path";
import { stat, open, rename, unlink, truncate, readFile, writeFile, appendFile, copyFile, mkdir, rmdir, readdir, createReadStream, createWriteStream, access, constants as fsConstants } from "fs";
import { promisify } from "util";
import { createInterface } from "readline";
const [statAsync, openAsync, renameAsync, unlinkAsync, truncateAsync, readFileAsync, writeFileAsync, appendFileAsync, copyFileAsync, mkdirAsync, rmdirAsync, readdirAsync] = [ stat, open, rename, unlink, truncate, readFile, writeFile, appendFile, copyFile, mkdir, rmdir, readdir ].map(fsFunc => promisify(fsFunc));
const [visibleAsync, readableAsync, writableAsync, executableAsync] = [ fsConstants.F_OK, fsConstants.R_OK, fsConstants.W_OK, fsConstants.X_OK ].map(mode => path => new Promise(resolve => access(path, mode, error => resolve(!error))));
const nearestExistAsync = async path => {
  for (;path && !await visibleAsync(path); ) path = dirname(path);
  return path;
};
const createPathPrefixLock = rootPath => {
  rootPath = resolvePath(rootPath);
  return relativePath => {
    const absolutePath = resolvePath(rootPath, relativePath);
    if (!absolutePath.startsWith(rootPath)) throw new Error(`invalid relativePath: ${relativePath}`);
    return absolutePath;
  };
};
const createReadlineFromStreamAsync = (readStream, onLineSync) => new Promise((resolve, reject) => {
  const readlineInterface = createInterface({
    input: readStream,
    historySize: 0,
    crlfDelay: 1 / 0
  });
  readlineInterface.on("error", error => {
    readlineInterface.close();
    reject(error);
  });
  readlineInterface.on("close", () => {
    resolve();
  });
  readlineInterface.on("line", line => {
    onLineSync(line);
  });
});
const createReadlineFromFileAsync = (filePath, onLineSync) => createReadlineFromStreamAsync(createReadStream(filePath), onLineSync);
const trimPathDepth = (path, depth) => normalize(path).split(sep).slice(0, depth).join(sep);
const REGEXP_PATH_SEP_WIN32 = /\\/g;
const toPosixPath = path => path.replace(REGEXP_PATH_SEP_WIN32, "/");
export { createReadStream, createWriteStream, statAsync, openAsync, renameAsync, unlinkAsync, truncateAsync, readFileAsync, writeFileAsync, appendFileAsync, copyFileAsync, mkdirAsync, rmdirAsync, readdirAsync, visibleAsync, readableAsync, writableAsync, executableAsync, nearestExistAsync, createPathPrefixLock, createReadlineFromStreamAsync, createReadlineFromFileAsync, trimPathDepth, toPosixPath };