import { resolve } from "path";
import { createLogQueue } from "../data/LogQueue";
import { createDirectory } from "../file/File";
import { createSafeWriteStream } from "./SafeWrite";
const createSimpleLogger = ({queueLengthThreshold, ...extraOption}) => createLogQueue({
  outputStream: createSafeWriteStream(extraOption),
  queueLengthThreshold
});
const createLogger = async ({pathLogDirectory, getLogFileName = () => `${new Date().toISOString().replace(/\W/g, "-")}.log`, saveInterval = 3e4, splitInterval = 864e5, ...extraOption}) => {
  let logger;
  let saveToken;
  let splitToken;
  const reset = () => {
    end();
    const pathOutputFile = resolve(pathLogDirectory, getLogFileName());
    logger = createSimpleLogger({
      ...extraOption,
      pathOutputFile
    });
    saveToken = saveInterval && setInterval(save, saveInterval);
    splitToken = splitInterval && setTimeout(split, splitInterval);
  };
  const save = () => {
    logger && logger.save();
  };
  const split = () => {
    logger && reset();
  };
  const end = () => {
    logger && logger.end();
    saveToken && clearInterval(saveToken);
    splitToken && clearTimeout(splitToken);
    logger = null;
    saveToken = null;
    splitToken = null;
  };
  await createDirectory(pathLogDirectory);
  reset();
  return {
    add: (...args) => {
      logger && logger.add(args.join(" "));
    },
    save,
    split,
    end
  };
};
export { createSimpleLogger, createLogger };