import { resolve, dirname } from "path";
import { tryRequire } from "../../../env/tryRequire";
import { splitCamelCase } from "../../../common/string";
import { string, number, boolean, integer, basicFunction, arrayLength, oneOf } from "../../../common/verify";
import { objectDeleteUndefined } from "../../../common/immutable/Object";
import { arraySplitChunk } from "../../../common/immutable/Array";
import { createOptionParser } from "./parser";
const throwError = message => {
  throw new Error(message);
};
const getPreset = (argumentCount, argumentListVerify = (() => {}), argumentListNormalize = (v => v), description = "", optional = !1) => ({
  argumentCount,
  argumentListNormalize,
  argumentListVerify,
  description,
  optional
});
const Preset = {
  Optional: {
    optional: !0
  },
  Path: {
    isPath: !0
  },
  Any: getPreset("0-", void 0, void 0, "any", !0),
  Toggle: getPreset("0-", void 0, () => [ !0 ], "set to enable", !0)
};
Object.assign(Preset, ...[ [ "String", string, argumentList => argumentList.map(String) ], [ "Number", number, argumentList => argumentList.map(Number) ], [ "Boolean", boolean, argumentList => argumentList.map(Boolean) ], [ "Integer", integer, argumentList => argumentList.map(parseInt) ], [ "Function", basicFunction, void 0 ] ].map(([typeName, verifyFunc, normalizeFunc]) => ({
  [`Single${typeName}`]: getPreset(1, argumentList => {
    arrayLength(argumentList, 1);
    verifyFunc(argumentList[0], `expect ${typeName}`);
  }, normalizeFunc),
  [`All${typeName}`]: getPreset("1-", argumentList => argumentList.forEach((v, i) => verifyFunc(v, `expect ${typeName} at #${i}`)), normalizeFunc)
})));
const parseCompact = (compactFormat, extraOption = {}) => {
  const [compactTag, ...descriptionList] = compactFormat.split("|");
  const [nameTag, presetTag = "", argumentCount] = compactTag.split(/\s*[\s\/]\s*/);
  const nameTagList = nameTag.split(",");
  const presetList = presetTag.split(",");
  return Object.assign({}, ...presetList.map(presetName => {
    presetName && !Preset[presetName] && throwError(`invalid presetName: ${presetName}`);
    return Preset[presetName];
  }).filter(Boolean), objectDeleteUndefined({
    name: nameTagList[0],
    shortName: nameTagList.find(nameTag => 1 === nameTag.length),
    aliasNameList: nameTagList.slice(1),
    argumentCount: argumentCount || void 0,
    description: descriptionList.join("|") || void 0
  }), Array.isArray(extraOption) ? {
    extendFormatList: extraOption
  } : extraOption);
};
Object.assign(Preset, {
  SinglePath: parseCompact("/SingleString,Path"),
  AllPath: parseCompact("/AllString,Path"),
  Config: parseCompact('config,c/SingleString,Optional|from ENV: set to "env"\nfrom JS/JSON file: set to "path/to/config.js|json"'),
  pickOneOf: selectList => {
    selectList.length <= 2 && throwError(`expect more to pick: ${selectList}`);
    return getPreset(1, argumentList => {
      arrayLength(argumentList, 1);
      oneOf(argumentList[0], selectList);
    }, void 0, `one of:\n  ${arraySplitChunk(selectList, 4).map(v => v.join(" ")).join("\n  ")}`);
  },
  parseCompact,
  parseCompactList: (...args) => args.map(compactFormat => Array.isArray(compactFormat) ? parseCompact(...compactFormat) : parseCompact(compactFormat))
});
Object.entries(Preset).forEach(([key, value]) => {
  const compactName = splitCamelCase(key).map(string => string.charAt(0)).join("");
  Preset[compactName] = value;
});
const getOptionalFormatFlag = (...formatNameList) => optionMap => !formatNameList.some(formatName => Boolean(optionMap[formatName]));
const getOptionalFormatValue = (formatName, ...valueList) => optionMap => {
  const format = optionMap[formatName];
  return format && !valueList.includes(format.argumentList[0]);
};
const parseOptionMap = async ({parseCLI, parseENV, parseCONFIG, processOptionMap}) => {
  const optionMapCLI = optionMapResolvePath(parseCLI(process.argv.slice(2)), process.cwd());
  const config = optionMapCLI.config && optionMapCLI.config.argumentList[0];
  return processOptionMap({
    ...config ? "env" === config ? optionMapResolvePath(parseENV(process.env), process.cwd()) : optionMapResolvePath(parseCONFIG(tryRequire(resolve(process.cwd(), config)) || throwError(`failed to load config: ${config}`)), dirname(resolve(process.cwd(), config))) : null,
    ...optionMapCLI
  });
};
const optionMapResolvePath = (optionMap, pathRoot) => {
  Object.values(optionMap).forEach(({format: {isPath}, argumentList}) => isPath && argumentList.forEach((v, i) => argumentList[i] = resolve(pathRoot, v)));
  return optionMap;
};
const createOptionGetter = optionMap => {
  const tryGet = name => optionMap[name] && optionMap[name].argumentList;
  const get = (name, argumentCount) => {
    const argumentList = tryGet(name);
    argumentList || throwError(`expect option ${name}`);
    void 0 !== argumentCount && argumentList.length !== argumentCount && throwError(`expect option ${name} with ${argumentCount} value, get ${argumentList.length}`);
    return argumentList;
  };
  return {
    optionMap,
    tryGet,
    tryGetFirst: name => optionMap[name] && optionMap[name].argumentList[0],
    get,
    getFirst: name => get(name, 1)[0]
  };
};
const prepareOption = optionConfig => {
  const {parseCLI, parseENV, parseCONFIG, processOptionMap, formatUsage} = createOptionParser(optionConfig);
  return {
    parseOption: async () => createOptionGetter(await parseOptionMap({
      parseCLI,
      parseENV,
      parseCONFIG,
      processOptionMap
    })),
    formatUsage
  };
};
export { Preset, getOptionalFormatFlag, getOptionalFormatValue, parseOptionMap, createOptionGetter, prepareOption };