import { openSync, write, writeSync, closeSync } from "fs";
import { rethrowError } from "../../common/error";
const createSafeWriteStream = ({pathOutputFile, flag = "w", mode = 438, onError = rethrowError}) => {
  let fileDescriptor = openSync(pathOutputFile, flag, mode);
  let writingQueue = [];
  let pendingQueue = [];
  const onWriteError = error => error ? onError(error) : writingQueue.length = 0;
  const doWrite = string => fileDescriptor && write(fileDescriptor, string, onWriteError);
  return {
    write: writeString => {
      if (writeString) {
        pendingQueue.push(writeString);
        if (0 === writingQueue.length) {
          [writingQueue, pendingQueue] = [ pendingQueue, [] ];
          process.nextTick(doWrite, writingQueue.join(""));
        }
      }
    },
    end: () => {
      const remainQueue = [ ...writingQueue, ...pendingQueue ];
      0 !== remainQueue.length && writeSync(fileDescriptor, remainQueue.join(""));
      closeSync(fileDescriptor);
      fileDescriptor = null;
      pendingQueue.length = writingQueue.length = 0;
    }
  };
};
export { createSafeWriteStream };