import { readFileSync } from "fs";
import { runInThisContext } from "vm";
import { tryRequireResolve } from "../env/tryRequire";
import { fetchLikeRequest } from "./net";
import { readFileAsync } from "./file/function";
const DR_BROWSER_FILE_PATH = () => [ "dr-js/library/Dr.browser.js", `${__dirname}/../library/Dr.browser.js`, `${__dirname}/Dr.browser.js`, "Dr.browser.js" ].reduce((o, path) => o || tryRequireResolve(path), null);
let cache;
const DR_BROWSER_SCRIPT_TAG = () => {
  void 0 === cache && (cache = `<script>${readFileSync(DR_BROWSER_FILE_PATH())}<\/script>`);
  return cache;
};
const loadRemoteScript = async uri => {
  const scriptString = await (await fetchLikeRequest(uri)).text();
  return runInThisContext(scriptString, {
    filename: uri,
    displayErrors: !0
  });
};
const loadLocalScript = async filePath => {
  const scriptString = String(await readFileAsync(filePath));
  return runInThisContext(scriptString, {
    filename: filePath,
    displayErrors: !0
  });
};
const loadScript = uri => uri.includes("://") ? loadRemoteScript(uri) : loadLocalScript(uri);
const loadRemoteJSON = async uri => (await fetchLikeRequest(uri)).json();
const loadLocalJSON = async filePath => JSON.parse(await readFileAsync(filePath));
const loadJSON = uri => uri.includes("://") ? loadRemoteJSON(uri) : loadLocalJSON(uri);
export { DR_BROWSER_FILE_PATH, DR_BROWSER_SCRIPT_TAG, loadRemoteScript, loadLocalScript, loadScript, loadRemoteJSON, loadLocalJSON, loadJSON };