import { clock } from "../../../common/time";
import { time as formatTime } from "../../../common/format";
const responderEnd = store => {
  if (store.response.finished) return;
  const {error} = store.getState();
  !store.response.headersSent && store.response.writeHead(error ? 500 : 400);
  store.response.end();
};
const responderEndWithStatusCode = (store, {statusCode = 500, headerMap}) => {
  if (!store.response.finished) {
    !store.response.headersSent && store.response.writeHead(statusCode, headerMap);
    store.response.end();
  }
};
const responderEndWithRedirect = (store, {statusCode = 302, redirectUrl}) => {
  if (!store.response.finished) {
    !store.response.headersSent && store.response.writeHead(statusCode, {
      location: redirectUrl
    });
    store.response.end();
  }
};
const responderSetHeaderCacheControlImmutable = store => {
  store.response.setHeader("cache-control", "max-age=315360000, public, immutable");
};
const DEFAULT_DESCRIBE_REQUEST = ({url, method, headers: {host}, socket: {remoteAddress, remotePort}}) => `[${method}] ${host || ""}${url} (${remoteAddress}:${remotePort})`;
const createResponderLog = ({log, describeRequest = DEFAULT_DESCRIBE_REQUEST}) => store => {
  const userAgentString = store.request.headers["user-agent"] || "no-user-agent";
  log(describeRequest(store.request), "[UA]", userAgentString);
};
const createResponderLogEnd = ({log, describeRequest = DEFAULT_DESCRIBE_REQUEST}) => store => {
  const {statusCode} = store.response;
  const {time, error} = store.getState();
  const timeString = formatTime(clock() - time);
  log(describeRequest(store.request), error ? `[ERROR|${statusCode}|${timeString}] ${error.stack || error}` : `[END|${statusCode}|${timeString}]`);
};
const createResponderSetHeaderHSTS = ({protocol}) => "https:" !== protocol ? () => {} : store => {
  store.response.setHeader("strict-transport-security", "max-age=31536000; includeSubDomains; preload");
};
const createResponderHostMapper = ({hostResponderMap, responderDefault}) => store => {
  const responder = hostResponderMap[(store.request.headers.host || "").toLowerCase()] || responderDefault;
  return responder && responder(store);
};
export { responderEnd, responderEndWithStatusCode, responderEndWithRedirect, responderSetHeaderCacheControlImmutable, createResponderLog, createResponderLogEnd, createResponderSetHeaderHSTS, createResponderHostMapper };