import { catchAsync } from "../../../common/error";
import { createCacheMap } from "../../../common/data/CacheMap";
import { responderEndWithStatusCode } from "./Common";
const DEFAULT_RESPONDER_DENY = (store, limitLeft) => responderEndWithStatusCode(store, {
  statusCode: limitLeft <= 0 ? 429 : 400
});
const DEFAULT_GET_REQUEST_KEY = store => `${store.request.socket.remoteAddress}`;
const GET_DEFAULT_CACHE_MAP = () => createCacheMap({
  valueSizeSumMax: 32768,
  eventHub: null
});
const createResponderRateLimit = ({responderNext, responderDeny = DEFAULT_RESPONDER_DENY, getRequestKey = DEFAULT_GET_REQUEST_KEY, limitCount = 32, expireTime = 6e5, rateCacheMap = GET_DEFAULT_CACHE_MAP()}) => store => {
  const cacheKey = getRequestKey(store);
  let limitLeft = rateCacheMap.get(cacheKey);
  void 0 === limitLeft && (limitLeft = limitCount);
  if (limitLeft <= 0) return responderDeny(store, limitLeft);
  rateCacheMap.set(cacheKey, limitLeft - 1, 1, Date.now() + expireTime);
  return responderNext(store, limitLeft);
};
const createResponderCheckRateLimit = ({checkFunc, responderNext, responderDeny = DEFAULT_RESPONDER_DENY, getRequestKey = DEFAULT_GET_REQUEST_KEY, limitCount = 32, expireTime = 6e5, rateCacheMap = GET_DEFAULT_CACHE_MAP()}) => async store => {
  const cacheKey = getRequestKey(store);
  let limitLeft = rateCacheMap.get(cacheKey);
  void 0 === limitLeft && (limitLeft = limitCount);
  if (limitLeft <= 0) return responderDeny(store, limitLeft);
  const {error, result: isCheckPass} = await catchAsync(checkFunc, store, limitLeft);
  if (isCheckPass) return responderNext(store, limitLeft);
  rateCacheMap.set(cacheKey, limitLeft - 1, 1, Date.now() + expireTime);
  return responderDeny(store, limitLeft, error);
};
export { createResponderRateLimit, createResponderCheckRateLimit };