import { isBasicObject } from "../../../common/check";
import { createTreeDepthFirstSearch } from "../../../common/data/Tree";
import { parseRouteToMap, findRouteFromMap, getRouteParamAny as getRouteMapParamAny, getRouteParam as getRouteMapParam } from "../../../common/module/RouteMap";
import { BASIC_EXTENSION_MAP } from "../../../common/module/MIME";
import { COMMON_LAYOUT, COMMON_STYLE } from "../commonHTML";
import { responderSendBufferCompress, prepareBufferData } from "./Send";
const METHOD_MAP = {
  GET: "/GET",
  POST: "/POST",
  PUT: "/PUT",
  PATCH: "/PATCH",
  DELETE: "/DELETE",
  HEAD: "/HEAD",
  OPTIONS: "/OPTIONS",
  CONNECT: "/CONNECT",
  TRACE: "/TRACE"
};
const appendRouteMap = (routeMap, route = "/", method = "GET", routeResponder) => {
  if (Array.isArray(route)) return route.reduce((o, v) => appendRouteMap(routeMap, v, method, routeResponder), routeMap);
  if (Array.isArray(method)) return method.reduce((o, v) => appendRouteMap(routeMap, route, v, routeResponder), routeMap);
  const {routeNode, paramNameList} = parseRouteToMap(routeMap, route);
  const methodTag = METHOD_MAP[method];
  if (!methodTag) throw new Error(`invalid method [${method}] for: ${route}`);
  if (routeNode[methodTag]) throw new Error(`duplicate method [${method}] for: ${route}`);
  if ("function" != typeof routeResponder) throw new Error(`invalid responder for: ${route}`);
  routeNode[methodTag] = {
    route,
    paramNameList,
    routeResponder
  };
  return routeMap;
};
const createRouteMap = configList => configList.reduce((o, [route, method, routeResponder]) => appendRouteMap(o, route, method, routeResponder), {});
const createResponderRouter = ({routeMap, baseUrl = "", getMethodUrl = createGetMethodUrl(new URL(baseUrl))}) => store => {
  const {method, url} = getMethodUrl(store);
  const methodTag = METHOD_MAP[method];
  if (void 0 === methodTag) return;
  const routeDate = findRouteFromMap(routeMap, url.pathname);
  if (void 0 === routeDate) return;
  const {routeNode, paramValueList} = routeDate;
  if (void 0 === routeNode[methodTag]) return;
  const {route, paramNameList, routeResponder} = routeNode[methodTag];
  const paramMap = paramNameList.reduce((o, paramName, index) => {
    o[paramName] = paramValueList[index];
    return o;
  }, {});
  return routeResponder(store, store.setState({
    method,
    url,
    route,
    paramMap
  }));
};
const createGetMethodUrl = baseUrlObject => ({request: {method, url}}) => ({
  method,
  url: new URL(url.replace(REGEXP_URL_REPLACE, "/"), baseUrlObject)
});
const REGEXP_URL_REPLACE = /\/\//g;
const getRouteParamAny = store => getRouteMapParamAny(store.getState());
const getRouteParam = (store, paramName) => getRouteMapParam(store.getState(), paramName);
const describeRouteMap = routeMap => {
  const routeInfoList = [];
  const methodValueSet = new Set(Object.values(METHOD_MAP));
  routeMapDepthFirstSearch([ "ROOT", routeMap ], ([key, {route}]) => {
    methodValueSet.has(key) && routeInfoList.push({
      method: key,
      route
    });
  });
  return routeInfoList;
};
const routeMapDepthFirstSearch = createTreeDepthFirstSearch(([key, routeNode]) => isBasicObject(routeNode) && Object.entries(routeNode));
const createResponderRouteListHTML = ({getRouteMap, extraBodyList}) => {
  let bufferData;
  return async store => {
    void 0 === bufferData && (bufferData = await prepareBufferData(Buffer.from(getRouteListHTML(getRouteMap(), extraBodyList)), BASIC_EXTENSION_MAP.html));
    return responderSendBufferCompress(store, bufferData);
  };
};
const getRouteListHTML = (routeMap, extraBodyList = []) => COMMON_LAYOUT([ COMMON_STYLE(), "<style>body { align-items: start; }</style>" ], [ "<h2>Route List</h2>", "<table>", ...describeRouteMap(routeMap).map(({method, route}) => `<tr><td><b>${method}</b></td><td>${"/GET" === method ? `<a href="${route}">${route}</a>` : route}</td></tr>`), "</table>", ...extraBodyList ]);
export { METHOD_MAP, createRouteMap, createResponderRouter, appendRouteMap, getRouteParamAny, getRouteParam, describeRouteMap, createResponderRouteListHTML };