import { WEBSOCKET_VERSION, WEBSOCKET_EVENT, getRespondKey } from "./function";
import { createWebSocket } from "./WebSocket";
const DEFAULT_ON_UPGRADE_REQUEST = (webSocket, request, bodyHeadBuffer) => webSocket.doCloseSocket();
const enableWebSocketServer = ({server, onUpgradeRequest = DEFAULT_ON_UPGRADE_REQUEST, frameLengthLimit}) => {
  const webSocketSet = new Set();
  server.on("upgrade", async (request, socket, bodyHeadBuffer) => {
    const webSocket = createWebSocket({
      socket,
      frameLengthLimit,
      isMask: !1,
      shouldPing: !0
    });
    const {responseKey} = ((webSocket, request) => {
      const requestKey = request.headers["sec-websocket-key"];
      const version = parseInt(request.headers["sec-websocket-version"]);
      if (!requestKey || version !== WEBSOCKET_VERSION || "GET" !== request.method || "websocket" !== request.headers.upgrade.toLowerCase()) return webSocket.doCloseSocket(new Error("invalid upgrade request"));
      webSocket.origin = request.headers.origin;
      webSocket.isSecure = Boolean(request.socket.authorized || request.socket.encrypted);
      webSocket.protocolList = (request.headers["sec-websocket-protocol"] || "").split(/, */);
      return {
        responseKey: getRespondKey(requestKey)
      };
    })(webSocket, request);
    if (webSocket.isClosed()) return;
    const protocol = await onUpgradeRequest(webSocket, request, bodyHeadBuffer);
    if (!webSocket.isClosed()) {
      if (!protocol) return webSocket.doCloseSocket(new Error("no selected protocol"));
      ((webSocket, protocol, responseKey) => {
        if (webSocket.getReadyState() !== webSocket.CONNECTING) throw new Error(`error readyState ${webSocket.getReadyState()}`);
        if (protocol && !webSocket.protocolList.includes(protocol)) throw new Error(`unexpected protocol ${protocol}`);
        webSocket.protocol = protocol;
        webSocket.open();
        webSocket.socket.write([ "HTTP/1.1 101 Switching Protocols", "upgrade: websocket", "connection: upgrade", `sec-websocket-accept: ${responseKey}`, protocol && `sec-websocket-protocol: ${protocol}`, "\r\n" ].filter(Boolean).join("\r\n"));
      })(webSocket, protocol, responseKey);
      if (!webSocket.isClosed()) {
        webSocketSet.add(webSocket);
        webSocket.on(WEBSOCKET_EVENT.CLOSE, () => {
          webSocketSet.delete(webSocket);
        });
      }
    }
  });
  return webSocketSet;
};
export { enableWebSocketServer };