import { catchAsync, catchSync } from "../../common/error";
const EXIT_LISTENER_LIST = [ {
  event: "exit",
  listener: code => exitListenerSync({
    eventType: "exit",
    code
  })
}, {
  event: "uncaughtException",
  listener: error => exitListenerAsync({
    eventType: "uncaughtException",
    error
  })
}, {
  event: "unhandledRejection",
  listener: (error, promise) => exitListenerAsync({
    eventType: "unhandledRejection",
    error,
    promise
  })
}, ...[ "SIGINT", "SIGHUP", "SIGQUIT", "SIGTERM" ].map(event => ({
  event,
  listener: () => exitListenerAsync({
    eventType: "signal",
    signalEventType: event
  })
})) ];
const listenerSyncSet = new Set();
const listenerAsyncSet = new Set();
const exitListenerSync = event => {
  for (const listenerSync of Array.from(listenerSyncSet)) catchSync(listenerSync, event);
};
const exitListenerAsync = async event => {
  unbindExitListener();
  for (const listenerAsync of Array.from(listenerAsyncSet)) await catchAsync(listenerAsync, event);
  exitListenerSync(event);
  process.exit(event.code || (event.error ? -1 : 0));
};
let isBind = !1;
const bindExitListener = () => {
  isBind = !0;
  EXIT_LISTENER_LIST.forEach(({event, listener}) => process.on(event, listener));
};
const unbindExitListener = () => {
  isBind = !1;
  EXIT_LISTENER_LIST.forEach(({event, listener}) => process.off(event, listener));
};
const clearExitListener = () => {
  listenerSyncSet.clear();
  listenerAsyncSet.clear();
  unbindExitListener();
};
const addExitListenerSync = (...args) => {
  args.forEach(listenerSync => listenerSyncSet.add(listenerSync));
  !isBind && bindExitListener();
};
const addExitListenerAsync = (...args) => {
  args.forEach(listenerAsync => listenerAsyncSet.add(listenerAsync));
  !isBind && bindExitListener();
};
const deleteExitListenerSync = (...args) => {
  args.forEach(listenerSync => listenerSyncSet.delete(listenerSync));
};
const deleteExitListenerAsync = (...args) => {
  args.forEach(listenerAsync => listenerAsyncSet.delete(listenerAsync));
};
export { clearExitListener, addExitListenerSync, addExitListenerAsync, deleteExitListenerSync, deleteExitListenerAsync };