import { setTimeoutAsync } from "../../common/time";
import { autoEllipsis } from "../../common/string";
import { padTable } from "../../common/format";
import { createTreeDepthFirstSearch, createTreeBottomUpSearchAsync, prettyStringifyTree } from "../../common/data/Tree";
import { runQuiet } from "./Run";
const GET_PROCESS_LIST_ASYNC_MAP = {
  INIT: !1,
  linux: null,
  win32: null,
  darwin: null,
  android: null
};
const getProcessListAsync = () => {
  !1 === GET_PROCESS_LIST_ASYNC_MAP.INIT && (() => {
    const createGetProcessListAsync = (command, lineSeparator, keyList, valueProcessList) => async () => {
      const {promise, stdoutBufferPromise} = runQuiet({
        command
      });
      await promise;
      return ((outputString, lineSeparator, keyList = [], valueProcessList = []) => {
        const [titleLine, ...rowList] = outputString.split(lineSeparator);
        const colStartIndexList = (titleString => {
          let flipCharType = " " === titleString.charAt(0);
          let flipCount = 2;
          const colStartIndexList = [ 0 ];
          for (let index = 0, indexMax = titleString.length; index < indexMax; index++) if (flipCharType !== (" " === titleString.charAt(index))) {
            flipCharType = !flipCharType;
            if (0 == --flipCount) {
              colStartIndexList.push(index);
              flipCount = 2;
            }
          }
          return colStartIndexList;
        })(titleLine);
        if (colStartIndexList.length !== keyList.length) throw new Error(`title col mismatch: ${colStartIndexList.length}, expect: ${keyList.length}`);
        return rowList.map(rowString => rowString && ((rowString, colStartIndexList, keyList, valueProcessList) => {
          const itemMap = {};
          for (let index = 0, indexMax = colStartIndexList.length; index < indexMax; index++) itemMap[keyList[index]] = valueProcessList[index](rowString.slice(colStartIndexList[index], colStartIndexList[index + 1]));
          return itemMap;
        })(rowString, colStartIndexList, keyList, valueProcessList)).filter(Boolean);
      })((await stdoutBufferPromise).toString(), lineSeparator, keyList, valueProcessList);
    };
    const valueProcessString = string => String(string).trim();
    const valueProcessInteger = string => parseInt(string);
    const ProcessListLinux = [ "ps ax -ww -o pid,ppid,args", "\n", [ "pid", "ppid", "command" ], [ valueProcessInteger, valueProcessInteger, valueProcessString ] ];
    const ProcessListAndroid = [ "ps ax -o pid,ppid,args", ...ProcessListLinux.slice(1) ];
    const ProcessListWin32 = [ "WMIC PROCESS get Commandline,ParentProcessId,Processid", "\r\r\n", [ "command", "ppid", "pid" ], [ valueProcessString, valueProcessInteger, valueProcessInteger ] ];
    const getProcessListAsyncLinux = createGetProcessListAsync(...ProcessListLinux);
    const getProcessListAsyncAndroid = createGetProcessListAsync(...ProcessListAndroid);
    const getProcessListAsyncWin32 = createGetProcessListAsync(...ProcessListWin32);
    Object.assign(GET_PROCESS_LIST_ASYNC_MAP, {
      INIT: !0,
      linux: getProcessListAsyncLinux,
      win32: getProcessListAsyncWin32,
      darwin: getProcessListAsyncLinux,
      android: getProcessListAsyncAndroid
    });
  })();
  const getProcessListAsync = GET_PROCESS_LIST_ASYNC_MAP[process.platform];
  if (!getProcessListAsync) throw new Error(`unsupported platform: ${process.platform}`);
  return getProcessListAsync();
};
const PROCESS_LIST_SORT_MAP = {
  "pid++": (a, b) => a.pid - b.pid,
  "pid--": (a, b) => b.pid - a.pid,
  "ppid++": (a, b) => a.ppid - b.ppid || a.pid - b.pid,
  "ppid--": (a, b) => b.ppid - a.ppid || a.pid - b.pid
};
const sortProcessList = (processList, sortOrder = "pid--") => processList.sort(PROCESS_LIST_SORT_MAP[sortOrder]);
const toProcessPidMap = processList => processList.reduce((o, info) => {
  o[info.pid] = info;
  return o;
}, {});
const toProcessTree = processList => {
  const rootInfo = {
    pid: 0,
    ppid: -1,
    command: "ROOT"
  };
  const processMap = {
    0: rootInfo
  };
  const subTreeMap = {
    0: {}
  };
  for (const info of processList) {
    const {pid, ppid} = info;
    if (0 !== pid) {
      processMap[pid] = info;
      void 0 === subTreeMap[ppid] && (subTreeMap[ppid] = {});
      subTreeMap[ppid][pid] = info;
    }
  }
  processMap[rootInfo.pid] = rootInfo;
  Object.entries(subTreeMap).forEach(([ppid, subTree]) => {
    let info = processMap[ppid];
    info || (subTreeMap[(info = {
      pid: ppid,
      ppid: rootInfo.pid,
      command: ""
    }).ppid][info.pid] = info);
    info.subTree = subTree;
  });
  return rootInfo;
};
const isInfoMatch = ({pid, ppid, command}, info) => info && info.pid === pid && (void 0 === ppid || info.ppid === ppid) && (void 0 === command || info.command === command);
const findProcessPidMapInfo = (info, processPidMap) => isInfoMatch(info, processPidMap[info.pid]) ? processPidMap[info.pid] : void 0;
const findProcessTreeInfo = (info, processTree) => processTreeDepthFirstSearch(processTree, searchInfo => isInfoMatch(info, searchInfo));
const killProcessInfoAsync = async info => {
  const isExistAsync = async () => isPidExist(info.pid) && findProcessPidMapInfo(info, toProcessPidMap(await getProcessListAsync()));
  if (await isExistAsync()) {
    process.kill(info.pid);
    await setTimeoutAsync(500);
    if (await isExistAsync()) {
      process.kill(info.pid);
      await setTimeoutAsync(2e3);
      if (await isExistAsync()) {
        process.kill(info.pid, "SIGKILL");
        await setTimeoutAsync(4e3);
        if (await isExistAsync()) throw new Error(`failed to stop process, pid: ${info.pid}, ppid: ${info.ppid}, command: ${info.command}`);
      }
    }
  }
};
const killProcessTreeInfoAsync = async (processTreeInfo, isMatchPpid = !1) => {
  const killAsync = isMatchPpid ? killProcessInfoAsync : ({pid, command}) => killProcessInfoAsync({
    pid,
    command
  });
  await processTreeBottomUpSearchAsync(processTreeInfo, killAsync);
  await killAsync(processTreeInfo);
};
const getSubNodeListFunc = info => info.subTree && Object.values(info.subTree);
const processTreeDepthFirstSearch = createTreeDepthFirstSearch(getSubNodeListFunc);
const processTreeBottomUpSearchAsync = createTreeBottomUpSearchAsync(getSubNodeListFunc);
const getAllProcessStatusAsync = async outputMode => {
  const processList = await getProcessListAsync();
  return outputMode.startsWith("t") ? toProcessTree(processList) : sortProcessList(processList, outputMode);
};
const describeAllProcessStatusAsync = async outputMode => {
  const status = await getAllProcessStatusAsync(outputMode);
  if (outputMode.startsWith("t")) {
    const text = prettyStringifyProcessTree(status);
    return "tree-wide" !== outputMode && "tw" !== outputMode ? text.split("\n").map(line => autoEllipsis(line, 128, 96, 16)).join("\n") : text;
  }
  return padTable({
    table: [ [ "pid", "ppid", "command" ], ...status.map(({pid, ppid, command}) => [ pid, ppid, command ]) ]
  });
};
const prettyStringifyProcessTree = processRootInfo => {
  const resultList = [];
  const addLine = (prefix, {pid, command}) => resultList.push(`${`${pid}`.padStart(8, " ")} | ${prefix}${command || "..."}`);
  addLine("", {
    pid: "pid",
    command: "command"
  });
  prettyStringifyTree([ processRootInfo, -1, !1 ], ([info, level, hasMore]) => info.subTree && Object.values(info.subTree).map((subInfo, subIndex, {length}) => [ subInfo, level + 1, subIndex !== length - 1 ]), addLine);
  return resultList.join("\n");
};
const isPidExist = pid => {
  try {
    process.kill(pid, 0);
    return !0;
  } catch (error) {
    return !1;
  }
};
export { getProcessListAsync, sortProcessList, toProcessPidMap, findProcessPidMapInfo, toProcessTree, findProcessTreeInfo, killProcessInfoAsync, killProcessTreeInfoAsync, getAllProcessStatusAsync, describeAllProcessStatusAsync, isPidExist };